@@ -11,7 +11,7 @@ log = logging.getLogger(__name__)
 
 __author__ = 'Erik-Jan van Baaren'
 __all__ = ['ESClient']
-__version__ = (0, 2, 1)
+__version__ = (0, 3, 0)
 
 
 def get_version():
@@ -44,9 +44,9 @@ class ESClient:
         if self.es_url.endswith('/'):
             self.es_url = self.es_url[:-1]
 
-    """
-    Internal helper methods
-    """
+    #
+    # Internal helper methods
+    # 
 
     def _make_path(self, path_components):
         """Create path from components. Empty components will be
@@ -59,6 +59,19 @@ class ESClient:
             path = '/' + path
         return path
 
+    def _parse_json_response(self, response):
+        """Convert JSON response from ElasticSearch to a hierarchy of Python
+        objects and return that hierarchy.
+        
+        Throws an exception when parsing fails.
+        
+        """
+        try:
+            return json.loads(response)
+        except:
+            raise ESClientException("Unable to parse JSON response from "
+                                    "ElasticSearch")
+
     def check_result(self, results, key, value):
         """Check if key is an element of list, and check if that element
         is equal (==) to value.
@@ -128,22 +141,22 @@ class ESClient:
             self.send_request(request_type, path,
                               query_string_args=query_string_args)
         elif operation_type == "_count":
-            """ If both options were not used, there one more option left: no
-            query at all. A query is optional when counting, so we fire a
-            request to the URL without a query only in this specific case. """
+            # If both options were not used, there one more option left: no
+            # query at all. A query is optional when counting, so we fire a
+            # request to the URL without a query only in this specific case.
             self.send_request('GET', path)
         else:
             raise ESClientException("Mandatory query was not supplied")
 
         try:
-            return json.loads(self.last_response.text)
+            return self._parse_json_response(self.last_response.text)
         except:
             raise ESClientException("Was unable to parse the ElasticSearch "
             "response as JSON: \n%s", self.last_response.text)
 
-    """
-    The API methods
-    """
+    #
+    # The API methods
+    #
 
     def index(self, index, doctype, body, docid=None, op_type=None):
         """Index the supplied document.
@@ -235,7 +248,7 @@ class ESClient:
 
         path = self._make_path([index, doctype, docid])
         self.send_request('GET', path, query_string_args=args)
-        return json.loads(self.last_response.text)
+        return self._parse_json_response(self.last_response.text)
 
     def mget(self, index, doctype, ids, fields=None):
         """Perform a multi get.
@@ -261,7 +274,7 @@ class ESClient:
             docs.append(doc)
         body = {'docs': docs}
         self.send_request('GET', path, body=body)
-        return json.loads(self.last_response.text)
+        return self._parse_json_response(self.last_response.text)
 
     def delete(self, index, doctype, id):
         """Delete document from index.
@@ -272,7 +285,7 @@ class ESClient:
         path = self._make_path([index, doctype, id])
         self.send_request('DELETE', path)
         resp = json.loads(self.last_response.text)
-        return resp['found']
+        return self.check_result(resp, 'found', True)
 
     """
     Indices API
@@ -376,16 +389,20 @@ class ESClient:
     def status(self, indexes=['_all']):
         """Retrieve the status of one or more indices.
         
+        Returns the JSON response converted to a hierachy of Python objects.
+        
         """
         path = self._make_path([','.join(indexes), '_status'])
         self.send_request('GET', path)
-        return json.loads(self.last_response.text)
+        return self._parse_json_response(self.last_response.text)
 
     def flush(self, indexes=['_all'], refresh=False):
         """Flush one or more indexes.
         
         Flush frees memory from the index by flushing data to the index
-        storage and clearing the internal transaction log.
+        storage and clearing the internal transaction log. There is
+        usually no need to use this function manually though.
+        
         """
         path = self._make_path([','.join(indexes), '_flush'])
         args = {}
@@ -395,5 +412,40 @@ class ESClient:
         resp = json.loads(self.last_response.text)
         return self.check_result(resp, 'ok', True)
 
+    def get_mapping(self, indexes=['_all'], doctypes=[]):
+        """Get mapping(s).
+        
+        You can get mappings for multiple indexes and/or multiple
+        types.
+        
+        Arguments:
+        indexes -- optional list of indexes
+        types -- optional list of types
+
+        """
+
+        # TODO: find out if you can get a mapping for multiple indexes
+        # and multiple types at the same time
+        path = self._make_path([','.join(indexes), ','.join(doctypes),
+                                '_mapping'])
+        self.send_request('GET', path)
+        return self._parse_json_response(self.last_response.text)
+
+    def put_mapping(self, mapping, doctype, indexes=['_all']):
+        """Register a mapping definition for a specific type. You can
+        register a mapping for one index, multiple indexes or even
+        all indexes (the default).
+        
+        Arguments:
+        indexes -- optional list of indexes (defaults to _all)
+        type -- the type you want this mapping to apply to
+        mapping -- a hierachy of Python object that can be converted to
+        a JSON document
+
+        """
+        path = self._make_path([','.join(indexes), doctype, '_mapping'])
+        self.send_request('PUT', path=path, body=mapping)
+        return self._parse_json_response(self.last_response.text)
+        
 if __name__ == '__main__':
     print "This is a library, it is not intended to be started by itself."
